/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"


//*****************************************************************************
//*****************************************************************************
// Section: Included Files
//*****************************************************************************
//*****************************************************************************

S_DATA16 rxBuffer[USB_DEVICE_AUDIO_QUEUE_DEPTH_COMBINED][APP_NO_OF_SAMPLES_IN_A_USB_FRAME];

APP_DATA appData =
{
    /* Device Layer Handle  */
    .usbDevHandle = -1,

    /* USB Audio Instance index for this app object 0*/
    .audioInstance = 0,

     /* app state */
    .state = APP_STATE_INIT,

    /* device configured status */
    .isConfigured = false,

    /* Initialize active interface setting to 0. */
    .activeInterfaceAlternateSetting = APP_USB_SPEAKER_PLAYBACK_NONE,

    /* No Audio control in progress.*/
    .currentAudioControl = APP_USB_CONTROL_NONE
};

int8_t sampleCounter = 0; // Keeps track of number of samples played.
bool newEvenData = false; // TRUE means a fresh data is arrived in the Even Buffer
bool newOddData  = false;  // TRUE means a fresh data is arrived in the Even Buffer
bool evenIsBeingPlayed = false; // TRUE means the audio frame in the Even buffer is being played currently  
bool oddIsBeingPlayed = false; // TRUE means the audio frame in the Odd buffer is being played currently 
int16_t *nextSample;  // Points to the Sample to be played next. 
U_DATA16 temp;

void APP_InterruptTasks(void);
// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

//******************************************************************************
// APP_USBDeviceEventHandler()
//******************************************************************************
void APP_USBDeviceEventHandler(USB_DEVICE_EVENT event, 
                               void * pEventData, 
                               uintptr_t context )
{
    volatile USB_DEVICE_EVENT_DATA_CONFIGURED* configuredEventData;

    switch( event )
    {
        case USB_DEVICE_EVENT_RESET:
            break;

        case USB_DEVICE_EVENT_DECONFIGURED:
            // USB device is reset or device is de-configured.
            // This means that USB device layer is about to de-initialize
            // all function drivers. So close handles to previously opened
            // function drivers.

            // Also turn ON LEDs to indicate reset/de-configured state.
            /* Switch on red and orange, switch off green */
               LED1_On();//r
               LED2_On();//o
               LED3_Off();//g

            break;

        case USB_DEVICE_EVENT_CONFIGURED:
            /* check the configuration */
            configuredEventData = 
                     (USB_DEVICE_EVENT_DATA_CONFIGURED *)pEventData;
            if(configuredEventData->configurationValue == 1)
            {
                /* the device is in configured state */
                /* Switch on green and switch off red and orange */
                LED1_Off();//r
                LED2_Off();//o
                LED3_On();//g
    
                USB_DEVICE_AUDIO_EventHandlerSet(0,
                                                 APP_USBDeviceAudioEventHandler ,
                                                 (uintptr_t)NULL);

                /* mark that set configuration is complete */
                appData.isConfigured = true;
            }
            break;

        case USB_DEVICE_EVENT_SUSPENDED:
            /* Switch on green and orange, switch off red */
               LED1_Off();//r
               LED2_On();//o
               LED3_On();//g
            break;

        case USB_DEVICE_EVENT_RESUMED:
        case USB_DEVICE_EVENT_POWER_DETECTED:
            /* VBUS was detected. Notify USB stack about the event */
            USB_DEVICE_Attach (appData.usbDevHandle);
            break;

        case USB_DEVICE_EVENT_POWER_REMOVED:
            /* VBUS was removed. Notify USB stack about the event*/
            USB_DEVICE_Detach (appData.usbDevHandle);
        case USB_DEVICE_EVENT_ERROR:
        default:
            break;
    }
} //End APP_USBDeviceEventHandler()


//******************************************************************************
// APP_USBDeviceAudioEventHandler()
//******************************************************************************
//void __attribute__((tcm)) 
//     APP_USBDeviceAudioEventHandler(USB_DEVICE_AUDIO_INDEX iAudio,
void APP_USBDeviceAudioEventHandler(USB_DEVICE_AUDIO_INDEX iAudio,
                               USB_DEVICE_AUDIO_EVENT event ,
                               void * pData,
                               uintptr_t context)
{
    volatile USB_DEVICE_AUDIO_EVENT_DATA_INTERFACE_SETTING_CHANGED *interfaceInfo;
    volatile USB_DEVICE_AUDIO_EVENT_DATA_READ_COMPLETE *readEventData;

    uint8_t entityID;
    uint8_t controlSelector;

    if ( iAudio == 0 )
    {
        switch (event)
        {
            case USB_DEVICE_AUDIO_EVENT_INTERFACE_SETTING_CHANGED:
                /* We have received a request from USB host to change the Interface-
                   Alternate setting.*/
                interfaceInfo = (USB_DEVICE_AUDIO_EVENT_DATA_INTERFACE_SETTING_CHANGED *)pData;
                appData.activeInterfaceAlternateSetting = interfaceInfo->interfaceAlternateSetting;
                appData.state = APP_USB_INTERFACE_ALTERNATE_SETTING_RCVD;
            break;

            case USB_DEVICE_AUDIO_EVENT_READ_COMPLETE:
                readEventData = (USB_DEVICE_AUDIO_EVENT_DATA_READ_COMPLETE *)pData;
                //We have received an audio frame from the Host.
                //Now send this audio frame to Audio Codec for Playback.

                    if(readEventData->handle == appData.readTransferHandle1)
                    {
                        appData.isReadComplete1 = true;
                    }
                    else if(readEventData->handle == appData.readTransferHandle2)
                    {
                        appData.isReadComplete2 = true;
                    }
                    else
                    {
                        
                    }
                    appData.state = APP_PROCESS_DATA;
            break;

            case USB_DEVICE_AUDIO_EVENT_WRITE_COMPLETE:
            {
            }
            break;


            case USB_DEVICE_AUDIO_EVENT_CONTROL_SET_CUR:
                entityID = ((USB_AUDIO_CONTROL_INTERFACE_REQUEST*)pData)->entityID;
                if (entityID == APP_ID_FEATURE_UNIT)
                {
                   controlSelector = ((USB_AUDIO_FEATURE_UNIT_CONTROL_REQUEST*)pData)->controlSelector;
                   if (controlSelector == USB_AUDIO_MUTE_CONTROL)
                   {
                       //A control write transfer received from Host. Now receive data from Host.
                       USB_DEVICE_ControlReceive(appData.usbDevHandle, (void *) &(appData.dacMute), 1 );
                       appData.currentAudioControl = APP_USB_AUDIO_MUTE_CONTROL;
                   }
                }
                break;
            case USB_DEVICE_AUDIO_EVENT_CONTROL_GET_CUR:
                entityID = ((USB_AUDIO_CONTROL_INTERFACE_REQUEST*)pData)->entityID;
                if (entityID == APP_ID_FEATURE_UNIT)
                {
                   controlSelector = ((USB_AUDIO_FEATURE_UNIT_CONTROL_REQUEST*)pData)->controlSelector;
                   if (controlSelector == USB_AUDIO_MUTE_CONTROL)
                   {
                       /*Handle Get request*/
                       USB_DEVICE_ControlSend(appData.usbDevHandle, (void *)&(appData.dacMute), 1 );
                   }
                }
                break;
            case USB_DEVICE_AUDIO_EVENT_CONTROL_SET_MIN:
            case USB_DEVICE_AUDIO_EVENT_CONTROL_GET_MIN:
            case USB_DEVICE_AUDIO_EVENT_CONTROL_SET_MAX:
            case USB_DEVICE_AUDIO_EVENT_CONTROL_GET_MAX:
            case USB_DEVICE_AUDIO_EVENT_CONTROL_SET_RES:
            case USB_DEVICE_AUDIO_EVENT_CONTROL_GET_RES:
            case USB_DEVICE_AUDIO_EVENT_ENTITY_GET_MEM:
                /* Stall request */
                USB_DEVICE_ControlStatus (appData.usbDevHandle, USB_DEVICE_CONTROL_STATUS_ERROR);
            break;
            case USB_DEVICE_AUDIO_EVENT_CONTROL_TRANSFER_DATA_RECEIVED:
                USB_DEVICE_ControlStatus(appData.usbDevHandle, USB_DEVICE_CONTROL_STATUS_OK );
                if (appData.currentAudioControl == APP_USB_AUDIO_MUTE_CONTROL)
                {
                    appData.state = APP_MUTE_AUDIO_PLAYBACK;
                    appData.currentAudioControl = APP_USB_CONTROL_NONE;
                    //Handle Mute Control Here.
                }
            break;
            case  USB_DEVICE_AUDIO_EVENT_CONTROL_TRANSFER_DATA_SENT:
            break;
            default:
                SYS_ASSERT ( false , "Invalid callback" );
            break;
        } //end of switch ( callback )
    }//end of if  if ( iAudio == 0 )
}//end of function APP_AudioEventCallback

/******************************************************
 * Application Initialize. It is
 * called from the SYS_Initialized() function.
 ******************************************************/
void APP_Initialize (void)
{
    appData.isReadComplete1 = false;
    appData.isReadComplete2 = false;

    appData.readTransferHandle1 = USB_DEVICE_AUDIO_TRANSFER_HANDLE_INVALID;
    appData.readTransferHandle2 = USB_DEVICE_AUDIO_TRANSFER_HANDLE_INVALID;
    
    OCMP1_Enable();//DRV_OC0_Start();
    OCMP4_Enable();//DRV_OC1_Start();
    
    TMR2_CallbackRegister( (TMR_CALLBACK) APP_InterruptTasks, 0 );
    
//      DRV_TMR0_Start;
//      PLIB_OC_PulseWidth16BitSet(OC_ID_1,500);
//      while(1);
}

/**********************************************************
 * Application tasks routine. This function implements the
 * application state machine.
 ***********************************************************/
USB_DEVICE_AUDIO_RESULT audioErr1;
void APP_Tasks (void )
{
    switch(appData.state)
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        /* Open the device layer */
            appData.usbDevHandle = USB_DEVICE_Open( USB_DEVICE_INDEX_0,
                    DRV_IO_INTENT_READWRITE );

            if(appData.usbDevHandle != USB_DEVICE_HANDLE_INVALID)
            {
                /* Register a callback with device layer to get event notification (for end point 0) */
                USB_DEVICE_EventHandlerSet(appData.usbDevHandle, APP_USBDeviceEventHandler, 0);

                appData.state = APP_STATE_WAIT_FOR_CONFIGURATION;
            }
            else
            {
                /* The Device Layer is not ready to be opened. We should try
                 * again later. */
            }

        case APP_STATE_WAIT_FOR_CONFIGURATION:
            //Check if Host has configured the Device.
            if (appData.isConfigured == true)
            {
                appData.state = APP_IDLE;
            }
            break;

       case APP_SUBMIT_INITIAL_READ_REQUEST:
            if (appData.activeInterfaceAlternateSetting == APP_USB_SPEAKER_PLAYBACK_STEREO_48KHZ)
            {
                audioErr1 = USB_DEVICE_AUDIO_Read ( USB_DEVICE_INDEX_0 , &appData.readTransferHandle1, 1 , rxBuffer[0], 192 );
                audioErr1 = USB_DEVICE_AUDIO_Read ( USB_DEVICE_INDEX_0 , &appData.readTransferHandle2, 1 , rxBuffer[1], 192 );
                appData.state = APP_PROCESS_DATA;
            }
            else
            {
            }
        break;

       case APP_PROCESS_DATA:
       {
            if(appData.isReadComplete1 == true)
            {
                appData.isReadComplete1 = false;
                
			//If the code gets to this point, we have received a USB frame from the host 
   			//with Audio Data in the Even Buffer. 
                newOddData = true;
    			if (evenIsBeingPlayed == false)// Start the Audio Playback for the Odd buffer only if the Even buffer is not being played.   
    			{
    				oddIsBeingPlayed = true;
    				nextSample = (int16_t *)&rxBuffer[0][0].leftData;// load the start address of the Audio data for Playback.   
    				sampleCounter =0;
                    // Unmask the Timer2 or Output Compare 1(in case of PIC24F) interrupt to start-
                    TMR2_Start();//DRV_TMR0_Start();
    			}
            //Get ready for next packet (this will overwrite the old data)
                audioErr1 = USB_DEVICE_AUDIO_Read ( USB_DEVICE_INDEX_0 , &appData.readTransferHandle1, 1 , rxBuffer[0], 192 );
            }
            else if(appData.isReadComplete2 == true)
            {
                appData.isReadComplete2 = false;
                
   			//If the code gets to this point, we have received a USB frame from the host 
   			//with Audio Data in the Even Buffer. 
    			newEvenData = true; 
    			if (oddIsBeingPlayed == false)// Start the Audio Playback for the Even buffer only if the Odd buffer is not being played.   
    			{		
    				evenIsBeingPlayed = true;
    				nextSample = (int16_t *)&rxBuffer[1][0].leftData;// load the start address of the Audio data for Playback.   
    				sampleCounter =0;
                    // Unmask the Timer2 or Output Compare 1(in case of PIC24F) interrupt to start-
                    TMR2_Start();//DRV_TMR0_Start();   	
    			}
        	//Get ready for next packet (this will overwrite the old data)
                audioErr1 = USB_DEVICE_AUDIO_Read ( USB_DEVICE_INDEX_0 , &appData.readTransferHandle2, 1 , rxBuffer[1], 192 );
            }
            else
            {
                ;
            }
       }

       break;

        case APP_MUTE_AUDIO_PLAYBACK:
        {
            if (appData.activeInterfaceAlternateSetting == 0)
            {
                appData.state = APP_IDLE;
            }
            else if (appData.activeInterfaceAlternateSetting == 1)
            {
                if(appData.dacMute== false){
                    OCMP1_Enable();//DRV_OC0_Start();
                    OCMP4_Enable();//DRV_OC1_Start();                    
                    appData.state = APP_SUBMIT_INITIAL_READ_REQUEST;
                }else{
                    OCMP1_Disable();//DRV_OC0_Stop();
                    OCMP4_Disable();//DRV_OC1_Stop();
                    appData.state = APP_IDLE;
                }
            }
        }
        break;

        case APP_USB_INTERFACE_ALTERNATE_SETTING_RCVD:
        {
           if (appData.activeInterfaceAlternateSetting == APP_USB_SPEAKER_PLAYBACK_NONE)
            {
                appData.state = APP_IDLE;
            }
            else if(appData.activeInterfaceAlternateSetting == APP_USB_SPEAKER_PLAYBACK_STEREO_48KHZ)
            {
                appData.state =  APP_SUBMIT_INITIAL_READ_REQUEST;
            }
        }
        break;

        case APP_IDLE:
        {
            if (appData.activeInterfaceAlternateSetting == APP_USB_SPEAKER_PLAYBACK_NONE)
            {
            }
        }
        break;

        case APP_STATE_ERROR:
        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

/**********************************************************
 * Application InterruptTasks.
 * This function is called by the Timer2 interrupt.
 ***********************************************************/
void APP_InterruptTasks(void)  //called in system.c
{

		if (sampleCounter < APP_NO_OF_SAMPLES_IN_A_USB_FRAME)
		{
			//If the code gets to this point, we are in the middle of an audio playback. 
	        // The Smaples Received Host is PCM (i.e signed 16 bit integer). The PWM running at 48KHz supports only 8 bits/Sample. 
	        // So here we convert each 16 bits Sample into 8 bit Sample. Load the next sample to the PWM Duty Cycle Register. 
	        temp.leftData = (*nextSample) + 32768 ;
            OCMP1_CompareSecondaryValueSet(temp.leftData/64);
           // PLIB_OC_PulseWidth16BitSet(OC_ID_1,temp.leftData/64);           
            nextSample++;            
            temp.rightData = (*nextSample) + 32768 ;
            OCMP4_CompareSecondaryValueSet(temp.rightData/64);           
           // PLIB_OC_PulseWidth16BitSet(OC_ID_4,temp.rightData/64);
			nextSample++;

			sampleCounter++;  
		}
        else if (oddIsBeingPlayed)
		{ 
   			// If the code gets to this point, we just finished playing Audio Samples in the Odd Buffer.
   			oddIsBeingPlayed = 	false;
			newOddData = false;
			if (newEvenData)//Check if Audio Samples are waiting to be played in Even Buffer 
			{
				// We need to start the Playing the audio samples in the Odd buffer immediately. 
				temp.leftData = rxBuffer[1][0].leftData + 32768; // Load the first Sample.
                OCMP1_CompareSecondaryValueSet(temp.leftData/64);
              //  PLIB_OC_PulseWidth16BitSet(OC_ID_1,temp.leftData/64);
   				temp.rightData = rxBuffer[1][0].rightData + 32768; // Load the first Sample.
               OCMP4_CompareSecondaryValueSet(temp.rightData/64);
              //  PLIB_OC_PulseWidth16BitSet(OC_ID_4,temp.rightData/64);
				nextSample = (int16_t *)&rxBuffer[1][1].leftData;// Set the adress for the second sample.
				evenIsBeingPlayed = true; 
				sampleCounter = 1;
			}
		}
		else if (evenIsBeingPlayed)
		{
			// If the code gets to this point, we just finished playing Audio Samples in the Even Buffer.
			evenIsBeingPlayed =false; 
			newEvenData = false; 
			if (newOddData)//Check if Audio Samples are waiting to be played in Odd Buffer 
			{
				// We need to start the Playing the audio samples in the Odd buffer immediately. 
				temp.leftData = rxBuffer[0][0].leftData + 32768; // Load the first Sample.
                OCMP1_CompareSecondaryValueSet(temp.leftData/64);
               // PLIB_OC_PulseWidth16BitSet(OC_ID_1,temp.leftData/64);
                temp.leftData = rxBuffer[0][0].rightData + 32768; // Load the first Sample.
                OCMP4_CompareSecondaryValueSet(temp.rightData/64);
                //PLIB_OC_PulseWidth16BitSet(OC_ID_4,temp.rightData/64);
				nextSample = (int16_t *)&rxBuffer[0][1].leftData;// Set the adress for the second sample.
				oddIsBeingPlayed = true;
				sampleCounter = 1;
			}  
		}
		else 
		{
   			//There is no Audio data present in the Even or Odd Buffers. So stop audio play back.
            // mask timer interrupt
            TMR2_Stop();//DRV_TMR0_Stop();
			sampleCounter = 0;	 
		}					

}
/*******************************************************************************
 End of File
 */
